#!/usr/bin/python
# -*- coding: utf-8 -*-

import logging

INSERT = 'INSERT'
UPDATE = 'UPDATE'
IGNORE = 'IGNORE'
ERROR = 'ERROR'
CONFLICT = 'CONFLICT'
DELETE = 'DELETE'
SKIP = 'SKIP'
RETRY = 'RETRY'
NO_CHANGE = 'NO_CHANGE'
SUCCESS = 'SUCCESS'

UNDEFINED = 'UNDEFINED'

OP_PLOT_SYMBOLS = {
    SUCCESS: '$',
    INSERT: '+',
    UPDATE: '.',
    IGNORE: '_',
    ERROR: 'X',
    CONFLICT: '!',
    DELETE: '#',
    SKIP: '/',
    RETRY: 'R',
    NO_CHANGE: '='
}

OP_DEFAULT_LOGGING_LEVELS = {
    INSERT: logging.DEBUG,
    UPDATE: logging.DEBUG,
    IGNORE: logging.DEBUG,
    ERROR: logging.ERROR,
    CONFLICT: logging.ERROR,
    DELETE: logging.INFO,
    SKIP: logging.DEBUG,
    RETRY: logging.INFO,
    NO_CHANGE: logging.DEBUG
}


class OpCounter:

    def __init__(self):
        self.counters = {}

    def inc(self, op_code, inc=1):
        self.counters[op_code] = self.counters.get(op_code, 0) + inc

    def total(self):
        total = 0
        for op_code in self.counters.keys():
            total += self.counters[op_code]
        return total

    def __str__(self):
        s = ''

        for op_code in self.counters.keys():
            if len(s) > 0:
                s = s + '\n'
            s = s + '\t{}: {}'.format(op_code.ljust(10), str(self.counters[op_code]).rjust(4))
        return s


class Plotter:
    DEFAULT_MAX_COL = 100

    def __init__(self, op_plot_symbols=OP_PLOT_SYMBOLS, max_col=DEFAULT_MAX_COL):
        self.op_plot_symbols = op_plot_symbols
        self.max_col = max_col
        self.plot_count = 0

    def plot(self, op_code):
        print(self.op_plot_symbols.get(op_code, '?'), end='', flush=True)

    def print_line_head(self, op_count):
        if op_count % self.max_col == 0:
            print('\n{}'.format(self.plot_count).ljust(6) + ': ', end='', flush=True)
            self.plot_count += self.max_col

    def reset(self):
        self.plot_count = 0
